@@ -33,10 +33,17 @@
 #define ALL 1
 
 int exitcode;
+int cpu_compress;
+
+enum {
+	CPU_COMPRESS = 300,
+	OPT_VERSION,
+};
 
 static struct option opts[] = {
 	{"all", 0, 0, 'a'},
 	{"interleave", 1, 0, 'i' },
+	{"weighted-interleave", 1, 0, 'w' },
 	{"preferred", 1, 0, 'p' },
 	{"preferred-many", 1, 0, 'P' },
 	{"cpubind", 1, 0, 'c' },
@@ -59,20 +66,24 @@ static struct option opts[] = {
 	{"shmid", 1, 0, 'I'},
 	{"huge", 0, 0, 'u'},
 	{"touch", 0, 0, 'T'},
+        {"cpu-compress", 0, 0, CPU_COMPRESS },
 	{"verify", 0, 0, 'V'}, /* undocumented - for debugging */
+	{"version", 0, 0, OPT_VERSION },
 	{ 0 }
 };
 
 static void usage(void)
 {
 	fprintf(stderr,
-		"usage: numactl [--all | -a] [--balancing | -b] [--interleave= | -i <nodes>]\n"
-		"		[--preferred= | -p <node>] [--preferred-many= | -P <nodes>]\n"
+		"usage: numactl [--all | -a] [--balancing | -b]\n"
+		"               [--interleave= | -i <nodes>] [--weighted-interleave= | -w <nodes>]\n"
+		"               [--preferred= | -p <node>] [--preferred-many= | -P <nodes>]\n"
 		"               [--physcpubind= | -C <cpus>] [--cpunodebind= | -N <nodes>]\n"
 		"               [--membind= | -m <nodes>] [--localalloc | -l] command args ...\n"
 		"               [--localalloc | -l] command args ...\n"
 		"       numactl [--show | -s]\n"
-		"       numactl [--hardware | -H]\n"
+		"       numactl [--hardware | -H] [--cpu-compress]\n"
+		"       numactl [--version]\n"
 		"       numactl [--length | -L <length>] [--offset | -o <offset>] [--shmmode | -M <shmmode>]\n"
 		"               [--strict | -t]\n"
 		"               [--shmid | -I <id>] --shm | -S <shmkeyfile>\n"
@@ -80,7 +91,8 @@ static void usage(void)
 		"               [--huge | -u] [--touch | -T] \n"
 		"               memory policy [--dump | -d] [--dump-nodes | -D]\n"
 		"\n"
-		"memory policy is --interleave | -i, --preferred | -p, --membind | -m, --localalloc | -l\n"
+		"memory policy is --preferred | -p, --membind | -m, --localalloc | -l,\n"
+		"                 --interleave | -i, --weighted-interleave | -w\n"
 		"<nodes> is a comma delimited list of node numbers or A-B ranges or all.\n"
 		"Instead of a number a node can also be:\n"
 		"  netdev:DEV the node connected to network device DEV\n"
@@ -186,6 +198,7 @@ static void show(void)
 	printmask("nodebind", cpubind);
 	printmask("membind", membind);
 	printmask("preferred", preferred);
+	numa_bitmask_free(preferred);
 }
 
 static char *fmt_mem(unsigned long long mem, char *buf)
@@ -212,36 +225,74 @@ static void print_distances(int maxnode)
 		return;
 	}
 	printf("node distances:\n");
-	printf("node ");
+	printf("node  ");
 	for (i = 0; i <= maxnode; i++)
 		if (numa_bitmask_isbitset(numa_nodes_ptr, i))
-			printf("% 3d ", i);
+			printf("% 4d ", i);
 	printf("\n");
 	for (i = 0; i <= maxnode; i++) {
 		if (!numa_bitmask_isbitset(numa_nodes_ptr, i))
 			continue;
-		printf("% 3d: ", i);
+		printf("% 4d: ", i);
 		for (k = 0; k <= maxnode; k++)
 			if (numa_bitmask_isbitset(numa_nodes_ptr, i) &&
 			    numa_bitmask_isbitset(numa_nodes_ptr, k))
-				printf("% 3d ", numa_distance(i,k));
+				printf("% 4d ", numa_distance(i,k));
 		printf("\n");
 	}
 }
 
 static void print_node_cpus(int node)
 {
-	int i, err;
-	struct bitmask *cpus;
+        int i = 0, err, start, segment = 0, count = 0;
+        struct bitmask *cpus;
 
-	cpus = numa_allocate_cpumask();
-	err = numa_node_to_cpus(node, cpus);
-	if (err >= 0) {
-		for (i = 0; i < cpus->size; i++)
-			if (numa_bitmask_isbitset(cpus, i))
-				printf(" %d", i);
-	}
-	putchar('\n');
+        cpus = numa_allocate_cpumask();
+        err = numa_node_to_cpus(node, cpus);
+        if (err < 0) {
+                goto out;
+        }
+
+        while (i < cpus->size) {
+                if (!cpu_compress) {
+                        if (numa_bitmask_isbitset(cpus, i))
+                        printf(" %d", i);
+                        i++;
+                        continue;
+                }
+
+                start = -1;
+
+                // Find the start and end of a range of available CPUs.
+                while (i < cpus->size && numa_bitmask_isbitset(cpus, i)) {
+                        if (start == -1) start = i;
+                        i++;
+                }
+                if (start == -1) {
+                        i++;
+                        continue;
+                }
+                if (segment) {
+                        printf(",");
+                }
+
+                int end = i - 1;
+                count += (end - start) + 1;
+                if (start == end) {
+                        printf(" %d", start);
+                } else {
+                        printf(" %d-%d", start, end);
+                }
+                segment++;
+        }
+
+        if (!cpu_compress)
+                printf("\n");
+        else
+                printf(" (%d)\n", count);
+
+out:
+        numa_free_cpumask(cpus);
 }
 
 static void hardware(void)
@@ -422,7 +473,6 @@ static struct bitmask *numactl_parse_nodestring(char *s, int flag)
 int main(int ac, char **av)
 {
 	int c;
-	long node=-1;
 	char *end;
 	char shortopts[array_len(opts)*2 + 1];
 	struct bitmask *mask = NULL;
@@ -432,6 +482,8 @@ int main(int ac, char **av)
 	int do_dump = 0;
 	int parse_all = 0;
 	int numa_balancing = 0;
+	int do_hardware = 0;
+	int weighted_interleave = 0;
 
 	get_short_opts(opts,shortopts);
 	while ((c = getopt_long(ac, av, shortopts, opts, NULL)) != -1) {
@@ -441,12 +493,15 @@ int main(int ac, char **av)
 			exit(0);
 		case 'H': /* --hardware */
 			nopolicy();
-			hardware();
-			exit(0);
+			do_hardware = 1;
+			break;
 		case 'b': /* --balancing  */
 			nopolicy();
 			numa_balancing = 1;
 			break;
+		case 'w': /* --weighted-interleave */
+			weighted_interleave = 1;
+			/* fall-through - logic is the same as interleave */
 		case 'i': /* --interleave */
 			checknuma();
 			if (parse_all)
@@ -460,11 +515,18 @@ int main(int ac, char **av)
 
 			errno = 0;
 			did_node_cpu_parse = 1;
-			setpolicy(MPOL_INTERLEAVE);
+			if (weighted_interleave)
+				setpolicy(MPOL_WEIGHTED_INTERLEAVE);
+			else
+				setpolicy(MPOL_INTERLEAVE);
 			if (shmfd >= 0)
 				numa_interleave_memory(shmptr, shmlen, mask);
-			else
-				numa_set_interleave_mask(mask);
+			else {
+				if (weighted_interleave)
+					numa_set_weighted_interleave_mask(mask);
+				else
+					numa_set_interleave_mask(mask);
+			}
 			checkerror("setting interleave mask");
 			break;
 		case 'N': /* --cpunodebind */
@@ -548,7 +610,13 @@ int main(int ac, char **av)
 			did_node_cpu_parse = 1;
 			numa_set_bind_policy(0);
 			if (shmfd >= 0) {
-				numa_tonode_memory(shmptr, shmlen, node);
+				numa_tonode_memory(shmptr, shmlen, find_first(mask));
+				/* Correspond to numa_set_bind_policy function */
+				if (numa_has_preferred_many()) {
+					setpolicy(MPOL_PREFERRED_MANY);
+				} else {
+					setpolicy(MPOL_PREFERRED);
+				}
 			} else if (c == 'p') {
 				if (numa_bitmask_weight(mask) != 1)
 					usage();
@@ -649,11 +717,23 @@ int main(int ac, char **av)
 			check_all_parse(did_node_cpu_parse);
 			parse_all = 1;
 			break;
+		case CPU_COMPRESS:
+			cpu_compress = 1;
+			break;
+		case OPT_VERSION:
+			nopolicy();
+			printf("%s\n", VERSION);
+			exit(0);
 		default:
 			usage();
 		}
 	}
 
+	if (do_hardware) {
+		hardware();
+		exit(0);
+	}
+
 	numa_bitmask_free(mask);
 
 	av += optind;
