@@ -9,8 +9,8 @@ extern int shmflags;
 
 extern void dump_shm(void);
 extern void dump_shm_nodes(void);
-extern void attach_shared(char *);
-extern void attach_sysvshm(char *);
+extern void attach_shared(char *, char *);
+extern void attach_sysvshm(char *, char *);
 extern void verify_shm(int policy, struct bitmask *);
 
 /* in numactl.c */
