@@ -6,9 +6,6 @@ extern int read_sysctl(char *name);
 extern void complain(char *fmt, ...);
 extern void nerror(char *fmt, ...);
 
-/* defined in main module, but called by util.c */
-extern void usage(void);
-
 extern long memsize(char *s);
 extern int parse_policy(char *name, char *arg);
 extern void print_policies(void);
@@ -22,5 +19,5 @@ extern char *policy_name(int policy);
 #if HAVE_ATTRIBUTE_SYMVER
 #define SYMVER(a,b) __attribute__ ((symver (b)))
 #else
-#define SYMVER(a,b) __asm__ (".symver " #a "," #b);
+#define SYMVER(a,b) __asm__ (".symver " a "," b);
 #endif
